#include "carwash.h"

const int CarWash::INFINITY = 10000; 
const int CarWash::MAX_SIZE = 5;
const int CarWash::WASH_TIME = 10;

CarWash::CarWash()
{
    currentTime = 0;
    numberOfCars = 0;
    sumOfWaitingTimes = 0;
    nextDepartureTime = INFINITY;



} // default constructor


void CarWash::runSimulation()
{
    const string PROMPT =
        "\nPlease enter the next arrival time. The sentinel is  ";

    const int SENTINEL = 999;

    queue<Car> carQueue;
    int nextArrivalTime;

    cout << PROMPT << SENTINEL << endl;
    cin >> nextArrivalTime;
    while (nextArrivalTime != SENTINEL)
    {
        if (nextArrivalTime < nextDepartureTime)
        {
            processArrival(nextArrivalTime, carQueue);
            cout << PROMPT << SENTINEL << endl;
            cin >> nextArrivalTime;
        } // if
        else
            processDeparture(carQueue);
    } // while SENTINEL not reached

    // Wash any cars remaining on the carQueue.
    while (nextDepartureTime < INFINITY)
        processDeparture(carQueue);
} // runSimulation

// Postcondition: the car that arrived at nextArrivalTime has either been turned away --
//                if carQueue was full before this message was sent --
//                or has entered this CarWash.
void CarWash::processArrival(int nextArrivalTime, queue<Car>& carQueue)
{
    const string OVERFLOW = "Overflow";

    currentTime = nextArrivalTime;
    if (carQueue.size() == MAX_SIZE)
        cout << OVERFLOW << endl;
    else
    {
        numberOfCars++;
        if (nextDepartureTime == INFINITY)
            nextDepartureTime = currentTime + WASH_TIME;
        else
            carQueue.push (Car (nextArrivalTime));
    } // not an overflow
} // method processArrival

// Postcondition: a car has finished getting washed and carQueue, if non-empty, has been
//		     popped.
void CarWash::processDeparture(queue<Car>& carQueue) {

    int waitingTime;

    cout << "departure time = " << nextDepartureTime << endl;
    currentTime = nextDepartureTime;
    if (!carQueue.empty())
    {
        Car car = carQueue.front();
        carQueue.pop();
        waitingTime = currentTime - car.getArrivalTime();
        sumOfWaitingTimes += waitingTime;
        nextDepartureTime = currentTime + WASH_TIME;
    } // carQueue was not empty
    else
        nextDepartureTime = INFINITY;
} // method processDeparture


void CarWash::printResult() {

    const string NO_CARS_MESSAGE = "There were no cars in the car wash.";

    const string AVERAGE_WAITING_TIME_MESSAGE =
        "\nThe average waiting time, in minutes, was ";

    if (numberOfCars == 0)
        cout << NO_CARS_MESSAGE << endl;
    else
        cout << AVERAGE_WAITING_TIME_MESSAGE
             << ((double)sumOfWaitingTimes / numberOfCars)
             << endl;
} // method printResult


